package sun.security.provider.certpath;

import java.util.Iterator;
import java.util.Set;
import java.security.cert.PKIXReason;
import java.security.cert.CertPathValidatorException;
import java.util.StringJoiner;
import java.util.Collections;
import java.security.cert.PKIXCertPathChecker;
import java.security.cert.X509Certificate;
import java.util.List;
import java.security.cert.CertPath;
import sun.security.util.Debug;

class PKIXMasterCertPathValidator
{
    private static final Debug debug;
    
    PKIXMasterCertPathValidator() {
    }
    
    static void validate(CertPath certPath, List<X509Certificate> list, List<PKIXCertPathChecker> list2) throws CertPathValidatorException {
        int size = list.size();
        if (PKIXMasterCertPathValidator.debug != null) {
            PKIXMasterCertPathValidator.debug.println("--------------------------------------------------------------");
            PKIXMasterCertPathValidator.debug.println("Executing PKIX certification path validation algorithm.");
        }
        int i = 0;
        while (i < size) {
            X509Certificate x509Certificate = (X509Certificate)list.get(i);
            if (PKIXMasterCertPathValidator.debug != null) {
                PKIXMasterCertPathValidator.debug.println("Checking cert" + (i + 1) + " - Subject: " + x509Certificate.getSubjectX500Principal());
            }
            Set<String> set = x509Certificate.getCriticalExtensionOIDs();
            if (set == null) {
                set = Collections.emptySet();
            }
            if (PKIXMasterCertPathValidator.debug != null && !set.isEmpty()) {
                StringJoiner o = new StringJoiner(", ", "{", "}");
                Iterator<Object> iterator = set.iterator();
                while (iterator.hasNext()) {
                    o.add((String)iterator.next());
                }
                PKIXMasterCertPathValidator.debug.println("Set of critical extensions: " + o.toString());
            }
            for (int j = 0; j < list2.size(); ++j) {
                PKIXCertPathChecker pkixCertPathChecker = (PKIXCertPathChecker)list2.get(j);
                if (PKIXMasterCertPathValidator.debug != null) {
                    PKIXMasterCertPathValidator.debug.println("-Using checker" + (j + 1) + " ... [" + pkixCertPathChecker.getClass().getName() + "]");
                }
                if (i == 0) {
                    pkixCertPathChecker.init(false);
                }
                try {
                    pkixCertPathChecker.check(x509Certificate, set);
                    if (PKIXMasterCertPathValidator.debug != null) {
                        PKIXMasterCertPathValidator.debug.println("-checker" + (j + 1) + " validation succeeded");
                    }
                }
                catch (CertPathValidatorException ex) {
                    throw new CertPathValidatorException(ex.getMessage(), (ex.getCause() != null) ? ex.getCause() : ex, certPath, size - (i + 1), ex.getReason());
                }
            }
            if (!set.isEmpty()) {
                throw new CertPathValidatorException("unrecognized critical extension(s)", null, certPath, size - (i + 1), PKIXReason.UNRECOGNIZED_CRIT_EXT);
            }
            if (PKIXMasterCertPathValidator.debug != null) {
                PKIXMasterCertPathValidator.debug.println("\ncert" + (i + 1) + " validation succeeded.\n");
            }
            ++i;
        }
        if (PKIXMasterCertPathValidator.debug != null) {
            PKIXMasterCertPathValidator.debug.println("Cert path validation succeeded. (PKIX validation algorithm)");
            PKIXMasterCertPathValidator.debug.println("--------------------------------------------------------------");
        }
    }
    
    static {
        debug = Debug.getInstance("certpath");
    }
}
